//**************************************************************
// LiteCAD Application Demo
// MDI Child Form Unit
// harry200@gmail.com
//**************************************************************

unit child;

interface

uses
  Winapi.Windows, Vcl.Dialogs, System.Classes, System.SysUtils, System.UITypes, System.Math, System.StrUtils,
  Vcl.Forms, Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

type
  TFormChild = class(TForm)
    PanelLiteCAD: TPanel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure PanelLiteCADResize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    hwMain:   HWND;     // Handle Parent LiteCAD window (TPanel as window)

    procedure CreateLiteCAD;
    procedure SetProps;
    procedure ResetCommands;

  public
    hLcWnd:   IntPtr;   // Handle LiteCAD window
    hLcDrw:   IntPtr;   // Handle LiteCAD drawing

    function  FileOpen(const FName: String): Boolean;
    function  FileOpenRecent: Boolean;
    function  FileSave(const SaveType: Integer): Boolean;
    function  Proceed: Boolean;
    procedure WndFocus;
    procedure Ruler(const View: Boolean);
    procedure Magnifier;
    procedure Aerial;
  end;

    procedure GetCoord(const hEvent: IntPtr; const Wnd: IntPtr);
    procedure LiteCADKeyDown(const hEvent: IntPtr; const Wnd: IntPtr);
    procedure RefreshMRU(const FName: String);

implementation

{$R *.dfm}

uses main, litecad, RecentList, res;

// Create LiteCad wnd
procedure TFormChild.FormCreate(Sender: TObject);
begin
  CreateLiteCAD;
end;

// Set Drawing prop wnd and state of settings
procedure TFormChild.FormActivate(Sender: TObject);
begin
  lcWndSetProps(hLcWnd,FormMain.hLcProp);
  FormMain.ButtonOrto.Down      := lcPropGetBool(hLcWnd,LC_PROP_WND_ORTHO)    = LC_TRUE;    // Orto (F2)
  FormMain.ButtonOSnap.Down     := lcPropGetBool(hLcWnd,LC_PROP_WND_OSNAP)    = LC_TRUE;    // Object snap (F3)
  FormMain.ButtonGrid.Down      := lcPropGetBool(hLcWnd,LC_PROP_WND_GRIDSHOW) = LC_TRUE;    // Grid (F7, Ctrl+G)
  FormMain.ButtonPolar.Down     := lcPropGetBool(hLcWnd,LC_PROP_WND_PTRACK)   = LC_TRUE;    // Polar tracking (F8)
  FormMain.ButtonSnap.Down      := lcPropGetBool(hLcWnd,LC_PROP_WND_GRIDSNAP) = LC_TRUE;    // Grid snap (F9)
  FormMain.ButtonLineWidth.Down := lcPropGetInt(hLcWnd, LC_PROP_WND_LWMODE)   = LC_LW_REAL; // LineWidth
  WndFocus
end;

// Cancel currently active command, de-select all entities, redraw drawing
procedure TFormChild.FormDeactivate(Sender: TObject);
begin
  ResetCommands
end;

procedure TFormChild.FormShow(Sender: TObject);
begin
// WndFocus
end;

// Ask about save drawing on close action
procedure TFormChild.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Proceed
    then CanClose:=true
    else CanClose:=false
end;

// Correctly stops active processes in LiteCAD graphics window
procedure TFormChild.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  lcWndOnClose(hLcWnd);
  if FormMain.MDIChildCount<=1 then
    begin
      FormMain.LabelCoords.Caption:=EmptyStr;
      FormMain.LabelPixelSize.Caption:=EmptyStr;
      FormMain.LabelVisibleArea.Caption:=EmptyStr;
      lcDeleteProps(hLcWnd);
    end;
    Action:=caFree
end;

// Save Drawing if needed Return Success
function TFormChild.Proceed: Boolean;
  var
    FName: String;
    SaveType: ShortInt;
    f: Integer;
begin
  Result:=true;
  if (hLcDrw=0) or (lcPropGetBool(hLcDrw,LC_PROP_DRW_DIRTY)=0) then exit;
  FName:=lcPropGetStr(hLcDrw,LC_PROP_DRW_FILENAME);
  if FName=EmptyStr
    then SaveType:=LC_CMD_FILESAVEAS
    else SaveType:=LC_CMD_FILESAVE;
  if FName=EmptyStr then FName:=Caption;
  f:=MessageDlg(Format(MsgAskSaveChanges,[FName]),mtConfirmation,[mbYes,mbNo,mbCancel],0);
  WndFocus;
  case f of
       IDYES: if FileSave(SaveType)
                  then Result:=true   // Success save file, allow actions
                  else Result:=false; // Unsuccess save file, disable actions
        IDNO: Result:=true;
    IDCANCEL: Result:=false;
  end;
  // reset current commands if allow further actions
  if Result then ResetCommands
end;

// Changes the size and position of a graphics window
procedure TFormChild.PanelLiteCADResize(Sender: TObject);
begin
  lcWndResize(hLcWnd,0,0,PanelLiteCAD.Width,PanelLiteCAD.Height);
end;

// Cancel currently active command, de-select all entities, redraw drawing
procedure TFormChild.ResetCommands;
begin
  lcWndExeCommand(hLcWnd,LC_CMD_RESET,0)
end;

// Event processing
procedure LcEventProc (hEvent: IntPtr); stdcall;
  var
    Wnd: IntPtr;
    EvType: IntPtr;
begin
  EvType:=lcPropGetInt(hEvent,LC_PROP_EVENT_TYPE);
  Wnd:=lcPropGetHandle(hEvent,LC_PROP_EVENT_WND);
  case EvType of
//    LC_EVENT_ADDSTR:      lcStrFileLoad(PChar(LCADLangRU));          // Localization LC interface
    LC_EVENT_HELP:        Application.HelpContext(0);
    LC_EVENT_MOUSEMOVE:   GetCoord(hEvent, Wnd);                       // Coordinates to statusbar
    LC_EVENT_KEYDOWN:     LiteCADKeyDown(hEvent, Wnd);                 // ShortCut Key press
    LC_EVENT_RULERCORNER: FormMain.ActionHelpAboutExecute(FormMain.ActionHelpAbout); // About LC on Rulers corner click
//    LC_EVENT_ADDCMD:                                                 // Add custom commands
//      begin
//        hLcWnd:=lcPropGetHandle(hEvent,LC_PROP_EVENT_WND);
//        AddCustomCommands(hLcWnd);
//      end;
  end;
end;

// Create LC wnd
procedure TFormChild.CreateLiteCAD;
  var WndStyle: IntPtr;
begin
  hwMain:=PanelLiteCAD.Handle;
  lcEventSetProc(LC_EVENT_HELP,        @LcEventProc, 0,0);
  lcEventSetProc(LC_EVENT_MOUSEMOVE,   @LcEventProc, 0,0);
  lcEventSetProc(LC_EVENT_KEYDOWN,     @LcEventProc, 0,0);
  lcEventSetProc(LC_EVENT_RULERCORNER, @LcEventProc, 0,0);
  lcInitialize();
  WndStyle:=FormMain.WND_STYLE; //LC_WS_RULERS or LC_WS_VIEWTABS;
  hLcWnd:=lcCreateWindow(hwMain,WndStyle);
  hLcDrw:=lcCreateDrawing;
  lcEventsEnable(LC_TRUE);
  lcDrwNew(hLcDrw,PChar(EmptyStr),hLcWnd);
  SetProps;
  lcWndZoomRect(hLcWnd,0,0,0,0); // zoom extents
  lcPropPutBool(hLcDrw,LC_PROP_DRW_DIRTY,LC_FALSE); // Drawing was modified flag to false state
end;

// Load LiteCAD Properties and Set Global Properties
procedure TFormChild.SetProps;
begin
//  lcPropPutStr(0,LC_PROP_G_REGCODE, PChar(LCADRegCode));     // Registration Code
  lcPropPutStr(0,LC_PROP_G_MSGTITLE,PChar(LCADMsgTitle));    // Title
  lcPropPutStr(0,LC_PROP_G_HELPFILE,PChar(LCADHelpFile));    // HelpFile
  lcPropPutBool(0,LC_PROP_G_TABCMDWND,LC_TRUE);              // Display CMD window by pressing the Ctrl+Tab
  lcPropPutBool(0,LC_PROP_G_SAVECFG,LC_TRUE);                // Determines whether to save configuration or not
  lcPropPutBool(0,LC_PROP_G_UNDOMSG,LC_FALSE);               // Don`t show Undo/Redo empty message
  lcPropPutBool(0,LC_PROP_G_PANREDQUAL,LC_FALSE);            // Reduce draw quality when panning
  lcPropPutInt(0,LC_PROP_G_MINCHARSIZE,5);                   // Ranges 3..15. Below this value a text character will be drawn as a rectangle
  lcPropPutInt(hLcDrw,LC_PROP_DRW_COLORBACKM,RGB(0,0,0));    // Model Space window background color
  lcPropPutInt(hLcWnd,LC_PROP_WND_GRIDCOLOR, RGB(50,55,72));  // Color of grid line
  lcPropPutInt(hLcWnd,LC_PROP_WND_GRIDCOLOR2,RGB(73,79,105)); // Color of grid bold line
  lcPropPutBool(hLcWnd,LC_PROP_WND_SELECT,1);
  lcPropPutBool(hLcWnd,LC_PROP_WND_KBMOVE_ENABLE,LC_TRUE);
  lcPropPutBool(hLcWnd, LC_PROP_WND_RULERS,   Integer(FormMain.ActionViewRulers.Checked));// Rulers
  lcPropPutBool(hLcWnd, LC_PROP_WND_GRIDSNAP, Integer(FormMain.ButtonSnap.Down));     // Grid snap (F9)
  lcPropPutBool(hLcWnd, LC_PROP_WND_GRIDSHOW, Integer(FormMain.ButtonGrid.Down));     // Grid (F7, Ctrl+G)
  lcPropPutBool(hLcWnd, LC_PROP_WND_OSNAP,    Integer(FormMain.ButtonOSnap.Down));    // Object snap (F3)
  lcPropPutBool(hLcWnd, LC_PROP_WND_ORTHO,    Integer(FormMain.ButtonOrto.Down));     // Orto (F2)
  lcPropPutBool(hLcWnd, LC_PROP_WND_PTRACK,   Integer(FormMain.ButtonPolar.Down));    // Polar tracking (F8)
  lcPropPutInt(hLcWnd,  LC_PROP_WND_LWMODE,   Integer(FormMain.ButtonLineWidth.Down));// LineWidth
  lcPropPutInt(hLcWnd,  LC_PROP_WND_OSNAP,    FormMain.WND_OSNAPI);                   // Object snap mode
  lcPropPutFloat(hLcWnd,LC_PROP_WND_GRIDDX,   FormMain.WND_GRIDDX);                   // Distance between vertical grid lines
  lcPropPutFloat(hLcWnd,LC_PROP_WND_GRIDDY,   FormMain.WND_GRIDDY);                   // Distance between horizontal grid lines
  lcWndRedraw(hLcWnd);
end;

// Refresh Recent list MRU
procedure RefreshMRU(const FName: String);
begin  
  if (FName<>EmptyStr) and MatchText(ExtractFileExt(FName),FExtensions) then
    if MRUList.IndexOf(FName)>=0
      then ChangePositionFileInMRUList(FormMain.LCRibbon,FName,MRUList.IndexOf(FName))
      else AddFileToMRUList(FormMain.LCRibbon,FName);
end;

// Open File
function TFormChild.FileOpen(const FName: String): Boolean;
begin
  Result:=lcDrwLoad(hLcDrw,PChar(FName),hLcWnd)<>0;
  if Result then
    begin 
      Caption:=ExtractFileName(FName);
      FormMain.ButtonOrto.Down      := lcPropGetBool(hLcWnd,LC_PROP_WND_ORTHO)    = LC_TRUE;
      FormMain.ButtonOSnap.Down     := lcPropGetBool(hLcWnd,LC_PROP_WND_OSNAP)    = LC_TRUE;
      FormMain.ButtonGrid.Down      := lcPropGetBool(hLcWnd,LC_PROP_WND_GRIDSHOW) = LC_TRUE;
      FormMain.ButtonPolar.Down     := lcPropGetBool(hLcWnd,LC_PROP_WND_PTRACK)   = LC_TRUE;
      FormMain.ButtonSnap.Down      := lcPropGetBool(hLcWnd,LC_PROP_WND_GRIDSNAP) = LC_TRUE;
      FormMain.ButtonLineWidth.Down := lcPropGetInt(hLcWnd, LC_PROP_WND_LWMODE)   = LC_LW_REAL;
      RefreshMRU(FName);
    end;
end;

// Open Recent File as LiteCAD function
function TFormChild.FileOpenRecent: Boolean;
  var FName: String;
begin
  Result:=lcDrwLoad(hLcDrw,'<Recent>',hLcWnd)<>0;
  if Result then 
    begin
      FName:=lcPropGetStr(hLcDrw,LC_PROP_DRW_FILENAME);
      Caption:=ExtractFileName(FName);
      RefreshMRU(FName)
    end;
end;

// Save File
function TFormChild.FileSave(const SaveType: Integer): Boolean;
  var FName: String;
begin
  Result:=lcWndExeCommand(hLcWnd,SaveType,0)<>0;
  if Result then
    begin
      FName:=lcPropGetStr(hLcDrw,LC_PROP_DRW_FILENAME);
      Caption:=ExtractFileName(FName);
      RefreshMRU(FName)
    end;
end;

// Sets the keyboard focus to the window
procedure TFormChild.WndFocus;
begin
  lcWndSetFocus(hLcWnd);
end;

// Show/Hide window rulers (The window must be created with LC_WS_RULERS style flag)
procedure TFormChild.Ruler(const View: Boolean);
begin
  lcPropPutBool(hLcWnd,LC_PROP_WND_RULERS,Integer(View));
end;

// Show/Hide Magnifier at right-bottom corner
procedure TFormChild.Magnifier;
  var View: Boolean;
begin
  View:=lcPropGetBool(hLcWnd,LC_PROP_WND_MAGNIFIER)=LC_True;
  lcPropPutBool(hLcWnd,LC_PROP_WND_MAGNIFIER,Integer(not View))
end;

// Show/Hide Aerial view window
procedure TFormChild.Aerial;
begin
  lcPropPutBool(hLcWnd,LC_PROP_WND_NAVIGATOR,LC_TRUE)
end;

// Cursor coordinates
procedure GetCoord(const hEvent: IntPtr; const Wnd: IntPtr);
  var X,Y,PixSize,L,B,R,T,W,H: Double;
begin
  X:=SimpleRoundTo(lcPropGetFloat(hEvent,LC_PROP_EVENT_FLOAT1),-4);
  Y:=SimpleRoundTo(lcPropGetFloat(hEvent,LC_PROP_EVENT_FLOAT2),-4);
  PixSize:=SimpleRoundTo(lcPropGetFloat(Wnd,LC_PROP_WND_PIXELSIZE),-4);
  L:=SimpleRoundTo(lcPropGetFloat(Wnd,LC_PROP_WND_XMIN),-2);
  B:=SimpleRoundTo(lcPropGetFloat(Wnd,LC_PROP_WND_YMIN),-2);
  R:=SimpleRoundTo(lcPropGetFloat(Wnd,LC_PROP_WND_XMAX),-2);
  T:=SimpleRoundTo(lcPropGetFloat(Wnd,LC_PROP_WND_YMAX),-2);
  W:=R - L;
  H:=T - B;
  FormMain.LabelCoords.Caption:=Format(Caption_XY,[X,Y]);
  FormMain.LabelPixelSize.Caption:=Format(Caption_PixelSize,[PixSize]);
  FormMain.LabelVisibleArea.Caption:=Format(Caption_VisibleArea,[L,B,R,T,W,H])
end;

// lcEventReturnCode(0) - Used inside of application-defined event procedure to send an integer return value to LiteCAD
// F2,F3,F7,F8,F9 bind in LiteCAD, just process button down
procedure LiteCADKeyDown(const hEvent: IntPtr; const Wnd: IntPtr);
  var
    RetCode:  IntPtr;
    VirtKey:  Integer;
    CtrlKey:  Integer;
begin
  RetCode:=0;
  VirtKey :=lcPropGetInt(hEvent,LC_PROP_EVENT_INT1);
//  Flags   :=lcPropGetInt(hEvent,LC_PROP_EVENT_INT2);
//  ShiftKey:=lcPropGetInt(hEvent,LC_PROP_EVENT_INT3);
  CtrlKey :=lcPropGetInt(hEvent,LC_PROP_EVENT_INT4);
  case VirtKey of
    VK_F2:  FormMain.ButtonOrto.Down  := lcPropGetBool(Wnd,LC_PROP_WND_ORTHO)   =LC_FALSE; // Orto (F2)
    VK_F3:  FormMain.ButtonOSnap.Down := lcPropGetBool(Wnd,LC_PROP_WND_OSNAP)   =LC_FALSE; // Object snap (F3)
    VK_F7:  FormMain.ButtonGrid.Down  := lcPropGetBool(Wnd,LC_PROP_WND_GRIDSHOW)=LC_FALSE; // Grid (F7, Ctrl+G)
    VK_F8:  FormMain.ButtonPolar.Down := lcPropGetBool(Wnd,LC_PROP_WND_PTRACK)  =LC_FALSE; // Polar tracking (F8)
    VK_F9:  FormMain.ButtonSnap.Down  := lcPropGetBool(Wnd,LC_PROP_WND_GRIDSNAP)=LC_FALSE; // Grid snap (F9)
       71: if CtrlKey<>0 then
             begin
               FormMain.ButtonGrid.Down  := lcPropGetBool(Wnd,LC_PROP_WND_GRIDSHOW)=LC_FALSE;
               lcPropPutBool(Wnd,LC_PROP_WND_GRIDSHOW,Integer(FormMain.ButtonGrid.Down));
               lcWndRedraw(Wnd);
             end;
       48: if CtrlKey<>0 then FormMain.FullScreenMode;
      else RetCode:=0;
  end;
  lcEventReturnCode(RetCode);
  // lcEventReturnCode(0); // 0 - let LiteCAD to process the key
  // lcEventReturnCode(1); // 1 - don't let LiteCAD to process the key
end;

end.
